function lazyload() {
  const calculateSizes = (el) => {
    const width = el.parentElement.clientWidth;
    el.setAttribute('sizes', `${width}px`);
  };

  const load = (el) => {
    const { src, srcset } = el.dataset;
    el.setAttribute('src', src);
    el.setAttribute('srcset', srcset || '');
    el.addEventListener('load', () => {
      el.classList.add('lazyloaded');
    });
    if (el.srcset) {
      calculateSizes(el);
      window.addEventListener('resize', () => {
        calculateSizes(el);
      });
    }
  };

  const modern = (elements) => {
    const callback = (entries) => {
      entries.forEach((entry) => {
        if (entry.intersectionRatio > 0) {
          load(entry.target);
        }
      });
    };

    const observer = new IntersectionObserver(callback);

    for (let i = 0; i < elements.length; i += 1) {
      observer.observe(elements[i]);
    }
  };

  const isInViewport = (el) => {
    const rect = el.getBoundingClientRect();
    const windowHeight = window.innerHeight || document.documentElement.clientHeight;
    const windowWidth = window.innerWidth || document.documentElement.clientWidth;
    const vertInView = rect.top <= windowHeight && rect.top + rect.height >= 0;
    const horInView = rect.left <= windowWidth && rect.left + rect.width >= 0;

    return vertInView && horInView;
  };

  const handleScroll = (elements, lastPosition, callback) => {
    window.requestAnimationFrame(() => {
      callback(elements, lastPosition);
    });
  };

  const fallback = (elements, lastPosition) => {
    if (lastPosition === window.pageYOffset) {
      handleScroll(elements, window.pageYOffset, fallback);
      return false;
    }

    for (let i = 0; i < elements.length; i += 1) {
      if (isInViewport(elements[i])) load(elements[i]);
    }

    handleScroll(elements, window.pageYOffset, fallback);

    return false;
  };

  const elements = document.querySelectorAll('[data-lazyload="img"]');

  if (window.IntersectionObserver) {
    modern(elements);
  } else {
    fallback(elements, -1);
  }
}

export default lazyload;
